@@ -39,7 +39,7 @@ def device(a: AGNumeric):
 
 
 @dispatch
-def move_to_active_device(a: AGNumeric):
+def to_active_device(a: AGNumeric):
     return a
 
 
@@ -148,38 +148,48 @@ def maximum(a: Numeric, b: Numeric):
 
 
 @dispatch
-def min(a: Numeric, axis=None):
-    return anp.min(a, axis=axis)
+def min(a: Numeric, axis=None, squeeze=True):
+    return anp.min(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def max(a: Numeric, axis=None):
-    return anp.max(a, axis=axis)
+def argmin(a: Numeric, axis=None):
+    return anp.argmin(a, axis=axis)
 
 
 @dispatch
-def sum(a: Numeric, axis=None):
-    return anp.sum(a, axis=axis)
+def max(a: Numeric, axis=None, squeeze=True):
+    return anp.max(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def mean(a: Numeric, axis=None):
-    return anp.mean(a, axis=axis)
+def argmax(a: Numeric, axis=None):
+    return anp.argmax(a, axis=axis)
 
 
 @dispatch
-def std(a: Numeric, axis=None):
-    return anp.std(a, axis=axis, ddof=0)
+def sum(a: Numeric, axis=None, squeeze=True):
+    return anp.sum(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def all(a: Numeric, axis=None):
-    return anp.all(a, axis=axis)
+def mean(a: Numeric, axis=None, squeeze=True):
+    return anp.mean(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def any(a: Numeric, axis=None):
-    return anp.any(a, axis=axis)
+def std(a: Numeric, axis=None, squeeze=True):
+    return anp.std(a, axis=axis, ddof=0, keepdims=not squeeze)
+
+
+@dispatch
+def all(a: Numeric, axis=None, squeeze=True):
+    return anp.all(a, axis=axis, keepdims=not squeeze)
+
+
+@dispatch
+def any(a: Numeric, axis=None, squeeze=True):
+    return anp.any(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
@@ -210,6 +220,11 @@ def bvn_cdf(a: Numeric, b: Numeric, c: Numeric):
     return _bvn_cdf(a, b, c)
 
 
+@dispatch
+def where(condition: Numeric, a: Numeric, b: Numeric):
+    return anp.where(condition, a, b)
+
+
 @dispatch
 def sort(a: Numeric, axis=-1, descending=False):
     if descending:
@@ -224,3 +239,8 @@ def argsort(a: Numeric, axis=-1, descending=False):
         return anp.argsort(-a, axis=axis)
     else:
         return anp.argsort(a, axis=axis)
+
+
+@dispatch
+def quantile(a: Numeric, q: Numeric, axis=None):  # pragma: no cover
+    raise NotImplementedError("Function `quantile` is not available for AutoGrad.")
