@@ -2,6 +2,7 @@ from types import FunctionType
 from typing import Callable
 
 import tensorflow as tf
+import tensorflow_probability as tfp
 
 from . import dispatch, B, Numeric, TFNumeric
 from .custom import tensorflow_register
@@ -48,7 +49,7 @@ def device(a: TFNumeric):
 
 
 @dispatch
-def move_to_active_device(a: TFNumeric):
+def to_active_device(a: TFNumeric):
     return a
 
 
@@ -178,43 +179,61 @@ def maximum(a: Numeric, b: Numeric):
 
 
 @dispatch
-def min(a: Numeric, axis=None):
-    return tf.reduce_min(a, axis=axis)
+def min(a: Numeric, axis=None, squeeze=True):
+    return tf.reduce_min(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def max(a: Numeric, axis=None):
-    return tf.reduce_max(a, axis=axis)
+def argmin(a: Numeric, axis=None):
+    if axis is None:
+        # The default `None` reduces over the last dimension.
+        return tf.argmin(tf.reshape(a, -1), axis=0)
+    else:
+        return tf.argmin(a, axis=axis)
 
 
 @dispatch
-def sum(a: Numeric, axis=None):
-    return tf.reduce_sum(a, axis=axis)
+def max(a: Numeric, axis=None, squeeze=True):
+    return tf.reduce_max(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def mean(a: Numeric, axis=None):
-    return tf.reduce_mean(a, axis=axis)
+def argmax(a: Numeric, axis=None):
+    if axis is None:
+        # The default `None` reduces over the last dimension.
+        return tf.argmax(tf.reshape(a, -1), axis=0)
+    else:
+        return tf.argmax(a, axis=axis)
 
 
 @dispatch
-def std(a: Numeric, axis=None):
+def sum(a: Numeric, axis=None, squeeze=True):
+    return tf.reduce_sum(a, axis=axis, keepdims=not squeeze)
+
+
+@dispatch
+def mean(a: Numeric, axis=None, squeeze=True):
+    return tf.reduce_mean(a, axis=axis, keepdims=not squeeze)
+
+
+@dispatch
+def std(a: Numeric, axis=None, squeeze=True):
     if axis is None:
         axes = list(range(B.rank(a)))
     else:
         axes = [axis]
-    _, var = tf.nn.moments(a, axes=axes)
+    _, var = tf.nn.moments(a, axes=axes, keepdims=not squeeze)
     return tf.sqrt(var)
 
 
 @dispatch
-def all(a: Numeric, axis=None):
-    return tf.reduce_all(a, axis=axis)
+def all(a: Numeric, axis=None, squeeze=True):
+    return tf.reduce_all(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
-def any(a: Numeric, axis=None):
-    return tf.reduce_any(a, axis=axis)
+def any(a: Numeric, axis=None, squeeze=True):
+    return tf.reduce_any(a, axis=axis, keepdims=not squeeze)
 
 
 @dispatch
@@ -250,6 +269,11 @@ def _cond(condition: TFNumeric, f_true: FunctionType, f_false: FunctionType, *ar
     return tf.cond(condition, lambda: f_true(*args), lambda: f_false(*args))
 
 
+@dispatch
+def where(condition: Numeric, a: Numeric, b: Numeric):
+    return tf.where(condition, a, b)
+
+
 # If `Numeric` types are used here, this implementation is more specific than the
 # generic implementation, which will use TensorFlow unnecessarily.
 @dispatch
@@ -273,3 +297,8 @@ def argsort(a: Numeric, axis=-1, descending=False):
     else:
         direction = "ASCENDING"
     return tf.argsort(a, axis=axis, direction=direction)
+
+
+@dispatch
+def quantile(a: Numeric, q: Numeric, axis=None):
+    return tfp.stats.percentile(a, 100 * q, axis=axis, interpolation="linear")
