@@ -1,3 +1,4 @@
+import jax
 import jax.numpy as jnp
 import numpy as np
 import pytest
@@ -18,7 +19,7 @@ from .util import (
     BoolTensor,
     NaNTensor,
     Bool,
-    allclose,
+    approx,
     check_lazy_shapes,
 )
 
@@ -78,10 +79,15 @@ def test_isnan(check_lazy_shapes):
     check_function(B.isnan, (NaNTensor(2, 3),), {}, assert_dtype=False)
 
 
-def test_device_and_move_to_active_device(check_lazy_shapes):
+def test_device_and_to_active_device(check_lazy_shapes):
+    # Check moving a device to the CPU.
     for a in Tensor(2, 2).forms():
         assert "cpu" in str(B.device(a)).lower()
-        allclose(B.move_to_active_device(a), a)
+        approx(B.to_active_device(a), a)
+
+    # Check that numbers remain unchanged.
+    a = 1
+    assert B.to_active_device(a) is a
 
 
 @pytest.mark.parametrize("t", [tf.float32, torch.float32, jnp.float32])
@@ -97,14 +103,14 @@ def test_device_and_move_to_active_device(check_lazy_shapes):
         lambda t: B.randn(t, 10),
     ],
 )
-def test_device_placement(f, t, check_lazy_shapes):
+def test_on_device(f, t, check_lazy_shapes):
     # Check that explicit allocation on CPU works.
-    with B.device("cpu"):
+    with B.on_device("cpu"):
         f(t)
 
     # Check that allocation on a non-existing device breaks.
     with pytest.raises(Exception):
-        with B.device("magic-device"):
+        with B.on_device("magic-device"):
             f(t)
 
     # Reset the active device. This is still set to "magic-device" due to the above
@@ -112,26 +118,38 @@ def test_device_placement(f, t, check_lazy_shapes):
     B.Device.active_name = None
 
 
-def test_move_to_active_device_jax(check_lazy_shapes):
+def test_set_global_device(check_lazy_shapes):
+    assert B.Device.active_name == None
+    B.set_global_device("gpu")
+    assert B.Device.active_name == "gpu"
+    B.Device.active_name = None
+
+
+def test_to_active_device_jax(check_lazy_shapes):
     a = jnp.ones(2)
 
     # No device specified: should do nothing.
-    assert B.move_to_active_device(a) is a
+    assert B.to_active_device(a) is a
+
+    # Move to JAX device.
+    with B.on_device(jax.devices("cpu")[0]):
+        assert B.to_active_device(a) is not a
+        approx(B.to_active_device(a), a)
 
     # Move to CPU without identifier.
-    with B.device("cpu"):
-        assert B.move_to_active_device(a) is not a
-        allclose(B.move_to_active_device(a), a)
+    with B.on_device("cpu"):
+        assert B.to_active_device(a) is not a
+        approx(B.to_active_device(a), a)
 
     # Move to CPU with identifier. Also check that capitalisation does not matter.
-    with B.device("CPU:0"):
-        assert B.move_to_active_device(a) is not a
-        allclose(B.move_to_active_device(a), a)
+    with B.on_device("CPU:0"):
+        assert B.to_active_device(a) is not a
+        approx(B.to_active_device(a), a)
 
     # Give invalid syntax.
     B.Device.active_name = "::"
     with pytest.raises(ValueError):
-        B.move_to_active_device(a)
+        B.to_active_device(a)
     B.Device.active_name = None
 
 
@@ -188,7 +206,7 @@ def test_zero_one(f, check_lazy_shapes):
 
 def test_linspace(check_lazy_shapes):
     # Check correctness.
-    allclose(B.linspace(0, 1, 10), np.linspace(0, 1, 10, dtype=B.default_dtype))
+    approx(B.linspace(0, 1, 10), np.linspace(0, 1, 10, dtype=B.default_dtype))
 
     # Check consistency.
     check_function(
@@ -204,9 +222,9 @@ def test_linspace(check_lazy_shapes):
 
 def test_range(check_lazy_shapes):
     # Check correctness.
-    allclose(B.range(5), np.arange(5))
-    allclose(B.range(2, 5), np.arange(2, 5))
-    allclose(B.range(2, 5, 2), np.arange(2, 5, 2))
+    approx(B.range(5), np.arange(5))
+    approx(B.range(2, 5), np.arange(2, 5))
+    approx(B.range(2, 5, 2), np.arange(2, 5, 2))
 
     # Check various step sizes.
     for step in [1, 1.0, 0.25]:
@@ -306,7 +324,7 @@ def test_unary_positive(f, check_lazy_shapes):
 
 @pytest.mark.parametrize("a", [0, -1, 1])
 def test_softplus_correctness(a, check_lazy_shapes):
-    allclose(B.softplus(a), np.log(1 + np.exp(a)))
+    approx(B.softplus(a), np.log(1 + np.exp(a)))
 
 
 @pytest.mark.parametrize(
@@ -325,18 +343,95 @@ def test_binary_positive_first(f, check_lazy_shapes):
     check_function(f, (PositiveTensor(2, 3), Tensor(2, 3)))
 
 
-@pytest.mark.parametrize("f", [B.min, B.max, B.sum, B.mean, B.std, B.logsumexp])
-def test_reductions(f, check_lazy_shapes):
+@pytest.mark.parametrize(
+    "f, check_squeeze",
+    [
+        (B.min, True),
+        (B.max, True),
+        (B.sum, True),
+        (B.nansum, True),
+        (B.mean, True),
+        (B.nanmean, True),
+        (B.std, True),
+        (B.nanstd, True),
+        (B.logsumexp, True),
+        (B.argmin, False),
+        (B.argmax, False),
+    ],
+)
+def test_reductions(f, check_squeeze, check_lazy_shapes):
     check_function(f, (Tensor(),))
     check_function(f, (Tensor(2),))
-    check_function(f, (Tensor(2),), {"axis": Value(0)})
     check_function(f, (Tensor(2, 3),))
-    check_function(f, (Tensor(2, 3),), {"axis": Value(0, 1)})
+    check_function(f, (Tensor(2),), {"axis": Value(None, -1, 0)})
+    check_function(f, (Tensor(2, 3),), {"axis": Value(None, -1, 0, 1)})
+    if check_squeeze:
+        check_function(
+            f,
+            (Tensor(2),),
+            {"axis": Value(None, -1, 0), "squeeze": Value(True, False)},
+        )
+        check_function(
+            f,
+            (Tensor(2, 3),),
+            {"axis": Value(None, -1, 0, 1), "squeeze": Value(True, False)},
+        )
+
+
+@pytest.mark.parametrize(
+    "f, f_ref",
+    [(B.nansum, B.sum), (B.nanmean, B.mean), (B.nanstd, B.std)],
+)
+def test_nanreductions(f, f_ref, check_lazy_shapes):
+    # Check consistency.
+    check_function(f, (NaNTensor(),), contains_nans=False)
+    check_function(f, (NaNTensor(2),), contains_nans=False)
+    check_function(f, (NaNTensor(2, 3),), contains_nans=False)
+    check_function(
+        f,
+        (NaNTensor(2),),
+        {"axis": Value(None, -1, 0)},
+        contains_nans=False,
+    )
+    check_function(
+        f,
+        (NaNTensor(2, 3),),
+        {"axis": Value(None, -1, 0, 1)},
+        contains_nans=False,
+    )
+    check_function(
+        f,
+        (Tensor(2),),
+        {"axis": Value(None, -1, 0), "squeeze": Value(True, False)},
+        contains_nans=False,
+    )
+    check_function(
+        f,
+        (Tensor(2, 3),),
+        {"axis": Value(None, -1, 0, 1), "squeeze": Value(True, False)},
+        contains_nans=False,
+    )
+
+    # Check against reference.
+    for x in Tensor(2, 3).forms():
+        approx(f(x), f_ref(x))
+        for axis in [None, -1, 0, 1]:
+            for squeeze in [True, False]:
+                approx(
+                    f(x, axis=axis, squeeze=squeeze),
+                    f_ref(x, axis=axis, squeeze=squeeze),
+                )
 
 
 def test_logsumexp_correctness(check_lazy_shapes):
     mat = PositiveTensor(3, 4).np()
-    allclose(B.logsumexp(mat, axis=1), scipy.special.logsumexp(mat, axis=1))
+    approx(B.logsumexp(mat), scipy.special.logsumexp(mat))
+    for axis in [None, -1, 0, 1]:
+        for squeeze in [True, False]:
+            approx(
+                B.logsumexp(mat, axis=axis, squeeze=squeeze),
+                scipy.special.logsumexp(mat, axis=axis, keepdims=not squeeze),
+            )
 
 
 @pytest.mark.parametrize("f", [B.all, B.any])
@@ -372,6 +467,13 @@ def test_cond(check_lazy_shapes):
         check_function(f, (Tensor(), Tensor(4)))
 
 
+def test_where(check_lazy_shapes):
+    def f(v, x, y):
+        return B.where(v > 0, x, y)
+
+    check_function(f, (Tensor(2, 3), Tensor(2, 3), Tensor(2, 3)))
+
+
 def test_scan(check_lazy_shapes):
     # Check consistency by inputting various shapes for a simple scanning
     # function.
@@ -398,7 +500,7 @@ def test_scan(check_lazy_shapes):
     xs = Tensor(10, 3, 4)
     init_h = Tensor(3, 4)
     init_y = Tensor(3, 4)
-    allclose(
+    approx(
         B.scan(scan_f, xs.np(), init_h.np(), init_y.np()),
         B.scan(scan_f, xs.tf(), init_h.tf(), init_y.tf()),
     )
@@ -440,6 +542,23 @@ def test_argsort(check_lazy_shapes):
     )
 
 
+def test_quantile(check_lazy_shapes):
+    # AutoGrad does not support `quantile`, so we skip it in both tests.
+    check_function(
+        B.quantile,
+        (Tensor(2, 3, 4), PositiveTensor(5, upper=1)),
+        {"axis": Value(None, -1, 0, 1)},
+        skip=[B.AGNumeric],
+    )
+    for q in [0, 0.5, 1]:
+        check_function(
+            B.quantile,
+            (Tensor(2, 3, 4), Value(q)),
+            {"axis": Value(None, -1, 0, 1)},
+            skip=[B.AGNumeric],
+        )
+
+
 def test_to_numpy(check_lazy_shapes):
     check_function(B.to_numpy, (Tensor(),))
     check_function(B.to_numpy, (Tensor(4),))
